@@ -1,4 +1,4 @@
-/*	$OpenBSD: interface.c,v 1.60 2007/09/11 16:02:55 claudio Exp $ */
+/*	$OpenBSD: interface.c,v 1.63 2009/06/05 04:12:52 claudio Exp $ */
 
 /*
  * Copyright (c) 2005 Claudio Jeker <claudio@openbsd.org>
@@ -215,8 +215,6 @@ if_del(struct iface *iface)
 {
 	struct nbr	*nbr = NULL;
 
-	log_debug("if_del: interface %s", iface->name);
-
 	/* revert the demotion when the interface is deleted */
 	if ((iface->state & (IF_STA_MULTI | IF_STA_POINTTOPOINT)) == 0)
 		ospfe_demote_iface(iface, 1);
@@ -240,6 +238,9 @@ if_del(struct iface *iface)
 void
 if_init(struct ospfd_conf *xconf, struct iface *iface)
 {
+	struct ifreq	ifr;
+	u_int		rdomain;
+
 	/* init the dummy local neighbor */
 	iface->self = nbr_new(ospfe_router_id(), iface, 1);
 
@@ -250,6 +251,18 @@ if_init(struct ospfd_conf *xconf, struct iface *iface)
 
 	iface->fd = xconf->ospf_socket;
 
+	strlcpy(ifr.ifr_name, iface->name, sizeof(ifr.ifr_name));
+	if (ioctl(iface->fd, SIOCGIFRTABLEID, (caddr_t)&ifr) == -1)
+		rdomain = 0;
+	else {
+		rdomain = ifr.ifr_rdomainid;
+		if (setsockopt(iface->fd, IPPROTO_IP, SO_RDOMAIN,
+		    &rdomain, sizeof(rdomain)) == -1)
+			fatal("failed to set rdomain");
+	}
+	if (rdomain != xconf->rdomain)
+		fatalx("interface rdomain mismatch");
+
 	ospfe_demote_iface(iface, 0);
 }
 
@@ -324,11 +337,8 @@ if_act_start(struct iface *iface)
 	if (!((iface->flags & IFF_UP) &&
 	    (LINK_STATE_IS_UP(iface->linkstate) ||
 	    (iface->linkstate == LINK_STATE_UNKNOWN &&
-	    iface->media_type != IFT_CARP)))) {
-		log_debug("if_act_start: interface %s link down",
-		    iface->name);
+	    iface->media_type != IFT_CARP))))
 		return (0);
-	}
 
 	if (iface->media_type == IFT_CARP && iface->passive == 0) {
 		/* force passive mode on carp interfaces */
@@ -440,7 +450,7 @@ start:
 	}
 
 	if (dr == NULL) {
-		/* no designate router found use backup DR */
+		/* no designated router found use backup DR */
 		dr = bdr;
 		bdr = NULL;
 	}
