@@ -1,4 +1,4 @@
-/*	$OpenBSD: kroute.c,v 1.59 2008/07/24 18:46:59 claudio Exp $ */
+/*	$OpenBSD: kroute.c,v 1.71 2009/06/05 22:40:24 chris Exp $ */
 
 /*
  * Copyright (c) 2004 Esben Norby <norby@openbsd.org>
@@ -46,6 +46,7 @@ struct {
 	int			fib_sync;
 	int			fd;
 	struct event		ev;
+	u_int			rdomain;
 } kr_state;
 
 struct kroute_node {
@@ -68,7 +69,7 @@ int	kif_compare(struct kif_node *, struct kif_node *);
 int	kr_change_fib(struct kroute_node *, struct kroute *, int, int);
 int	kr_delete_fib(struct kroute_node *);
 
-struct kroute_node	*kroute_find(in_addr_t, u_int8_t);
+struct kroute_node	*kroute_find(in_addr_t, u_int8_t, u_int8_t);
 struct kroute_node	*kroute_matchgw(struct kroute_node *, struct in_addr);
 int			 kroute_insert(struct kroute_node *);
 int			 kroute_remove(struct kroute_node *);
@@ -87,7 +88,7 @@ struct kroute_node	*kroute_match(in_addr_t);
 int		protect_lo(void);
 u_int8_t	prefixlen_classful(in_addr_t);
 void		get_rtaddrs(int, struct sockaddr *, struct sockaddr **);
-void		if_change(u_short, int, struct if_data *);
+void		if_change(u_short, int, struct if_data *, struct sockaddr_dl *);
 void		if_newaddr(u_short, struct sockaddr_in *, struct sockaddr_in *,
 		    struct sockaddr_in *);
 void		if_announce(void *);
@@ -109,6 +110,9 @@ int
 kif_init(void)
 {
 	RB_INIT(&kit);
+	/* init also krt tree so that we can call kr_shutdown() */
+	RB_INIT(&krt);
+	kr_state.fib_sync = 0;	/* decoupled */
 
 	if (fetchifs(0) == -1)
 		return (-1);
@@ -117,12 +121,13 @@ kif_init(void)
 }
 
 int
-kr_init(int fs)
+kr_init(int fs, u_int rdomain)
 {
 	int		opt = 0, rcvbuf, default_rcvbuf;
 	socklen_t	optlen;
 
 	kr_state.fib_sync = fs;
+	kr_state.rdomain = rdomain;
 
 	if ((kr_state.fd = socket(AF_ROUTE, SOCK_RAW, 0)) == -1) {
 		log_warn("kr_init: socket");
@@ -150,8 +155,6 @@ kr_init(int fs)
 	kr_state.pid = getpid();
 	kr_state.rtseq = 1;
 
-	RB_INIT(&krt);
-
 	if (fetchtable() == -1)
 		return (-1);
 
@@ -186,13 +189,17 @@ kr_change_fib(struct kroute_node *kr, struct kroute *kroute, int krcount,
 					break;
 			}
 			nkn = kn->next;
-			if (i == krcount)
+			if (i == krcount) {
 				/* stale route */
 				if (kr_delete_fib(kn) == -1)
 					log_warnx("kr_delete_fib failed");
-			log_debug("kr_update_fib: before: %s%s",
-			    inet_ntoa(kn->r.nexthop),
-			    i == krcount ? " (deleted)" : "");
+				/*
+				 * if head element was removed we need to adjust
+				 * the head
+				 */
+				if (kr == kn)
+					kr = nkn;
+			}
 		}
 	}
 
@@ -212,10 +219,6 @@ kr_change_fib(struct kroute_node *kr, struct kroute *kroute, int krcount,
 					break;
 			}
 
-			log_debug("kr_update_fib: after : %s%s",
-			     inet_ntoa(kroute[i].nexthop),
-			     kn == NULL ? " (added)" : "");
-
 			if (kn != NULL)
 				/* nexthop already present, skip it */
 				continue;
@@ -236,6 +239,7 @@ kr_change_fib(struct kroute_node *kr, struct kroute *kroute, int krcount,
 		kn->r.prefixlen = kroute[i].prefixlen;
 		kn->r.nexthop.s_addr = kroute[i].nexthop.s_addr;
 		kn->r.flags = kroute[i].flags | F_OSPFD_INSERTED;
+		kn->r.priority = RTP_OSPF;
 		kn->r.ext_tag = kroute[i].ext_tag;
 		rtlabel_unref(kn->r.rtlabel);	/* for RTM_CHANGE */
 		kn->r.rtlabel = kroute[i].rtlabel;
@@ -259,32 +263,10 @@ kr_change(struct kroute *kroute, int krcount)
 
 	kroute->rtlabel = rtlabel_tag2id(kroute->ext_tag);
 
-	kr = kroute_find(kroute->prefix.s_addr, kroute->prefixlen);
-
-	if (kr != NULL) {
-		if (kr->r.flags & F_KERNEL) {
-			/* a non-ospf route already exists. not a problem */
-			if (!(kr->r.flags & F_BGPD_INSERTED)) {
-				do {
-					kr->r.flags |= F_OSPFD_INSERTED;
-					kr = kr->next;
-				} while (kr);
-				return (0);
-			}
-			/*
-			 * XXX as long as there is no multipath support in
-			 * bgpd this is safe else we end up in a bad situation.
-			 */
-			/*
-			 * ospf route has higher pref
-			 * - reset flags to the ospf ones
-			 * - use RTM_CHANGE
-			 * - zero out ifindex (this is no longer relevant)
-			 */
-			action = RTM_CHANGE;
-		} else if (kr->next == NULL)	/* single path OSPF route */
-			action = RTM_CHANGE;
-	}
+	kr = kroute_find(kroute->prefix.s_addr, kroute->prefixlen, RTP_OSPF);
+	if (kr != NULL && kr->next == NULL && krcount == 1)
+		/* single path OSPF route */
+		action = RTM_CHANGE;
 
 	return (kr_change_fib(kr, kroute, krcount, action));
 }
@@ -292,14 +274,9 @@ kr_change(struct kroute *kroute, int krcount)
 int
 kr_delete_fib(struct kroute_node *kr)
 {
-	if (!(kr->r.flags & F_OSPFD_INSERTED))
-		return (0);
-
-	if (kr->r.flags & F_KERNEL) {
-		/* remove F_OSPFD_INSERTED flag, route still exists in kernel */
-		kr->r.flags &= ~F_OSPFD_INSERTED;
-		return (0);
-	}
+	if (kr->r.priority != RTP_OSPF)
+		log_warn("kr_delete_fib: %s/%d has wrong priority %d",
+		    inet_ntoa(kr->r.prefix), kr->r.prefixlen, kr->r.priority);
 
 	if (send_rtmsg(kr_state.fd, RTM_DELETE, &kr->r) == -1)
 		return (-1);
@@ -315,8 +292,8 @@ kr_delete(struct kroute *kroute)
 {
 	struct kroute_node	*kr, *nkr;
 
-	if ((kr = kroute_find(kroute->prefix.s_addr, kroute->prefixlen)) ==
-	    NULL)
+	if ((kr = kroute_find(kroute->prefix.s_addr, kroute->prefixlen,
+	    RTP_OSPF)) == NULL)
 		return (0);
 
 	while (kr != NULL) {
@@ -348,10 +325,9 @@ kr_fib_couple(void)
 	kr_state.fib_sync = 1;
 
 	RB_FOREACH(kr, kroute_tree, &krt)
-		if (!(kr->r.flags & F_KERNEL))
-			for (kn = kr; kn != NULL; kn = kn->next) {
+		if (kr->r.priority == RTP_OSPF)
+			for (kn = kr; kn != NULL; kn = kn->next)
 				send_rtmsg(kr_state.fd, RTM_ADD, &kn->r);
-			}
 
 	log_info("kernel routing table coupled");
 }
@@ -365,12 +341,10 @@ kr_fib_decouple(void)
 	if (kr_state.fib_sync == 0)	/* already decoupled */
 		return;
 
-	RB_FOREACH(kr, kroute_tree, &krt) {
-		if (!(kr->r.flags & F_KERNEL))
-			for (kn = kr; kn != NULL; kn = kn->next) {
+	RB_FOREACH(kr, kroute_tree, &krt)
+		if (kr->r.priority == RTP_OSPF)
+			for (kn = kr; kn != NULL; kn = kn->next)
 				send_rtmsg(kr_state.fd, RTM_DELETE, &kn->r);
-			}
-	}
 
 	kr_state.fib_sync = 0;
 
@@ -381,6 +355,7 @@ kr_fib_decouple(void)
 void
 kr_dispatch_msg(int fd, short event, void *bula)
 {
+	/* XXX this is stupid */
 	dispatch_rtmsg();
 }
 
@@ -494,12 +469,14 @@ kr_redist_eval(struct kroute *kr, struct rroute *rr)
 	    (a >> IN_CLASSA_NSHIFT) == IN_LOOPBACKNET)
 		goto dont_redistribute;
 	/*
-	 * Consider networks with nexthop loopback as not redistributable.
+	 * Consider networks with nexthop loopback as not redistributable
+	 * unless it is a reject or blackhole route.
 	 */
-	if (kr->nexthop.s_addr == htonl(INADDR_LOOPBACK))
+	if (kr->nexthop.s_addr == htonl(INADDR_LOOPBACK) &&
+	    !(kr->flags & (F_BLACKHOLE|F_REJECT)))
 		goto dont_redistribute;
 
-	/* Should we redistrubute this route? */
+	/* Should we redistribute this route? */
 	if (!ospf_redistribute(kr, &metric))
 		goto dont_redistribute;
 
@@ -531,6 +508,10 @@ kr_redistribute(struct kroute_node *kh)
 	struct rroute		 rr;
 	int			 redistribute = 0;
 
+	/* only the highest prio route can be redistributed */
+	if (kroute_find(kh->r.prefix.s_addr, kh->r.prefixlen, RTP_ANY) != kh)
+		return;
+
 	bzero(&rr, sizeof(rr));
 	rr.metric = UINT_MAX;
 	for (kn = kh; kn; kn = kn->next)
@@ -590,6 +571,14 @@ kroute_compare(struct kroute_node *a, struct kroute_node *b)
 		return (-1);
 	if (a->r.prefixlen > b->r.prefixlen)
 		return (1);
+
+	/* if the priority is RTP_ANY finish on the first address hit */
+	if (a->r.priority == RTP_ANY || b->r.priority == RTP_ANY)
+		return (0);
+	if (a->r.priority < b->r.priority)
+		return (-1);
+	if (a->r.priority > b->r.priority)
+		return (1);
 	return (0);
 }
 
@@ -601,14 +590,27 @@ kif_compare(struct kif_node *a, struct kif_node *b)
 
 /* tree management */
 struct kroute_node *
-kroute_find(in_addr_t prefix, u_int8_t prefixlen)
+kroute_find(in_addr_t prefix, u_int8_t prefixlen, u_int8_t prio)
 {
 	struct kroute_node	s;
+	struct kroute_node	*kn, *tmp;
 
 	s.r.prefix.s_addr = prefix;
 	s.r.prefixlen = prefixlen;
+	s.r.priority = prio;
 
-	return (RB_FIND(kroute_tree, &krt, &s));
+	kn = RB_FIND(kroute_tree, &krt, &s);
+	if (kn && prio == RTP_ANY) {
+		tmp = RB_PREV(kroute_tree, &krt, kn);
+		while (tmp) {
+			if (kroute_compare(&s, tmp) == 0)
+				kn = tmp;
+			else 
+				break;
+			tmp = RB_PREV(kroute_tree, &krt, kn);
+		}
+	}
+	return (kn);
 }
 
 struct kroute_node *
@@ -638,7 +640,6 @@ kroute_insert(struct kroute_node *kr)
 		 * Multipath route, add at end of list and clone the
 		 * ospfd inserted flag.
 		 */
-		kr->r.flags |= krm->r.flags & F_OSPFD_INSERTED;
 		while (krm->next != NULL)
 			krm = krm->next;
 		krm->next = kr;
@@ -799,9 +800,14 @@ kif_update(u_short ifindex, int flags, struct if_data *ifd,
 {
 	struct kif_node		*kif;
 
-	if ((kif = kif_find(ifindex)) == NULL)
+	if ((kif = kif_find(ifindex)) == NULL) {
 		if ((kif = kif_insert(ifindex)) == NULL)
 			return (NULL);
+		kif->k.nh_reachable = (flags & IFF_UP) &&
+		    (LINK_STATE_IS_UP(ifd->ifi_link_state) ||
+		    (ifd->ifi_link_state == LINK_STATE_UNKNOWN &&
+		    ifd->ifi_type != IFT_CARP));
+	}
 
 	kif->k.flags = flags;
 	kif->k.link_state = ifd->ifi_link_state;
@@ -843,11 +849,12 @@ kroute_match(in_addr_t key)
 
 	/* we will never match the default route */
 	for (i = 32; i > 0; i--)
-		if ((kr = kroute_find(key & prefixlen2mask(i), i)) != NULL)
+		if ((kr = kroute_find(key & prefixlen2mask(i), i,
+		    RTP_ANY)) != NULL)
 			return (kr);
 
 	/* if we don't have a match yet, try to find a default route */
-	if ((kr = kroute_find(0, 0)) != NULL)
+	if ((kr = kroute_find(0, 0, RTP_ANY)) != NULL)
 			return (kr);
 
 	return (NULL);
@@ -928,13 +935,14 @@ get_rtaddrs(int addrs, struct sockaddr *sa, struct sockaddr **rti_info)
 }
 
 void
-if_change(u_short ifindex, int flags, struct if_data *ifd)
+if_change(u_short ifindex, int flags, struct if_data *ifd,
+    struct sockaddr_dl *sdl)
 {
 	struct kroute_node	*kr, *tkr;
 	struct kif		*kif;
 	u_int8_t		 reachable;
 
-	if ((kif = kif_update(ifindex, flags, ifd, NULL)) == NULL) {
+	if ((kif = kif_update(ifindex, flags, ifd, sdl)) == NULL) {
 		log_warn("if_change:  kif_update(%u)", ifindex);
 		return;
 	}
@@ -1035,10 +1043,12 @@ send_rtmsg(int fd, int action, struct kroute *kroute)
 	bzero(&hdr, sizeof(hdr));
 	hdr.rtm_version = RTM_VERSION;
 	hdr.rtm_type = action;
-	hdr.rtm_flags = RTF_PROTO2|RTF_MPATH;
 	hdr.rtm_priority = RTP_OSPF;
-	if (action == RTM_CHANGE)	/* force PROTO2 reset the other flags */
-		hdr.rtm_fmask = RTF_PROTO2|RTF_PROTO1|RTF_REJECT|RTF_BLACKHOLE;
+	hdr.rtm_tableid = kr_state.rdomain;	/* rtableid */
+	if (action == RTM_CHANGE)
+		hdr.rtm_fmask = RTF_REJECT|RTF_BLACKHOLE;
+	else
+		hdr.rtm_flags = RTF_MPATH;
 	hdr.rtm_seq = kr_state.rtseq++;	/* overflow doesn't matter */
 	hdr.rtm_msglen = sizeof(hdr);
 	/* adjust iovec */
@@ -1148,7 +1158,7 @@ fetchtable(void)
 	mib[3] = AF_INET;
 	mib[4] = NET_RT_DUMP;
 	mib[5] = 0;
-	mib[6] = 0;	/* rtableid */
+	mib[6] = kr_state.rdomain;	/* rtableid */
 
 	if (sysctl(mib, 7, NULL, &len, NULL, 0) == -1) {
 		log_warn("sysctl");
@@ -1169,7 +1179,7 @@ fetchtable(void)
 		rtm = (struct rt_msghdr *)next;
 		if (rtm->rtm_version != RTM_VERSION)
 			continue;
-		sa = (struct sockaddr *)(rtm + 1);
+		sa = (struct sockaddr *)(next + rtm->rtm_hdrlen);
 		get_rtaddrs(rtm->rtm_addrs, sa, rti_info);
 
 		if ((sa = rti_info[RTAX_DST]) == NULL)
@@ -1185,6 +1195,7 @@ fetchtable(void)
 		}
 
 		kr->r.flags = F_KERNEL;
+		kr->r.priority = rtm->rtm_priority;
 
 		switch (sa->sa_family) {
 		case AF_INET:
@@ -1193,10 +1204,12 @@ fetchtable(void)
 			sa_in = (struct sockaddr_in *)rti_info[RTAX_NETMASK];
 			if (rtm->rtm_flags & RTF_STATIC)
 				kr->r.flags |= F_STATIC;
+			if (rtm->rtm_flags & RTF_BLACKHOLE)
+				kr->r.flags |= F_BLACKHOLE;
+			if (rtm->rtm_flags & RTF_REJECT)
+				kr->r.flags |= F_REJECT;
 			if (rtm->rtm_flags & RTF_DYNAMIC)
 				kr->r.flags |= F_DYNAMIC;
-			if (rtm->rtm_flags & RTF_PROTO1)
-				kr->r.flags |= F_BGPD_INSERTED;
 			if (sa_in != NULL) {
 				if (sa_in->sin_len == 0)
 					break;
@@ -1225,7 +1238,7 @@ fetchtable(void)
 				break;
 			}
 
-		if (rtm->rtm_flags & RTF_PROTO2)  {
+		if (rtm->rtm_priority == RTP_OSPF)  {
 			send_rtmsg(kr_state.fd, RTM_DELETE, &kr->r);
 			free(kr);
 		} else {
@@ -1253,7 +1266,6 @@ fetchifs(u_short ifindex)
 	struct rt_msghdr	*rtm;
 	struct if_msghdr	 ifm;
 	struct ifa_msghdr	*ifam;
-	struct kif		*kif = NULL;
 	struct sockaddr		*sa, *rti_info[RTAX_MAX];
 
 	mib[0] = CTL_NET;
@@ -1287,17 +1299,8 @@ fetchifs(u_short ifindex)
 			memcpy(&ifm, next, sizeof(ifm));
 			sa = (struct sockaddr *)(next + rtm->rtm_hdrlen);
 			get_rtaddrs(ifm.ifm_addrs, sa, rti_info);
-
-			if ((kif = kif_update(ifm.ifm_index,
-			    ifm.ifm_flags, &ifm.ifm_data,
-			    (struct sockaddr_dl *)rti_info[RTAX_IFP])) == NULL)
-				fatal("fetchifs");
-
-			kif->nh_reachable = (kif->flags & IFF_UP) &&
-			    (LINK_STATE_IS_UP(ifm.ifm_data.ifi_link_state) ||
-			    (ifm.ifm_data.ifi_link_state ==
-			    LINK_STATE_UNKNOWN &&
-			    ifm.ifm_data.ifi_type != IFT_CARP));
+			if_change(ifm.ifm_index, ifm.ifm_flags, &ifm.ifm_data,
+			    (struct sockaddr_dl *)rti_info[RTAX_IFP]);
 			break;
 		case RTM_NEWADDR:
 			ifam = (struct ifa_msghdr *)rtm;
@@ -1332,7 +1335,7 @@ dispatch_rtmsg(void)
 	struct sockaddr_rtlabel	*label;
 	struct kroute_node	*kr, *okr;
 	struct in_addr		 prefix, nexthop;
-	u_int8_t		 prefixlen;
+	u_int8_t		 prefixlen, prio;
 	int			 flags, mpath;
 	u_short			 ifindex = 0;
 
@@ -1357,13 +1360,14 @@ dispatch_rtmsg(void)
 		flags = F_KERNEL;
 		nexthop.s_addr = 0;
 		mpath = 0;
+		prio = 0;
 
 		if (rtm->rtm_type == RTM_ADD || rtm->rtm_type == RTM_CHANGE ||
 		    rtm->rtm_type == RTM_DELETE) {
-			sa = (struct sockaddr *)(rtm + 1);
+			sa = (struct sockaddr *)(next + rtm->rtm_hdrlen);
 			get_rtaddrs(rtm->rtm_addrs, sa, rti_info);
 
-			if (rtm->rtm_tableid != 0)
+			if (rtm->rtm_tableid != kr_state.rdomain)
 				continue;
 
 			if (rtm->rtm_pid == kr_state.pid) /* caused by us */
@@ -1379,6 +1383,8 @@ dispatch_rtmsg(void)
 			if (rtm->rtm_flags & RTF_MPATH)
 				mpath = 1;
 #endif
+			prio = rtm->rtm_priority;
+
 			switch (sa->sa_family) {
 			case AF_INET:
 				prefix.s_addr =
@@ -1396,10 +1402,12 @@ dispatch_rtmsg(void)
 					    prefixlen_classful(prefix.s_addr);
 				if (rtm->rtm_flags & RTF_STATIC)
 					flags |= F_STATIC;
+				if (rtm->rtm_flags & RTF_BLACKHOLE)
+					flags |= F_BLACKHOLE;
+				if (rtm->rtm_flags & RTF_REJECT)
+					flags |= F_REJECT;
 				if (rtm->rtm_flags & RTF_DYNAMIC)
 					flags |= F_DYNAMIC;
-				if (rtm->rtm_flags & RTF_PROTO1)
-					flags |= F_BGPD_INSERTED;
 				break;
 			default:
 				continue;
@@ -1428,8 +1436,8 @@ dispatch_rtmsg(void)
 				continue;
 			}
 
-			if ((okr = kroute_find(prefix.s_addr, prefixlen)) !=
-			    NULL) {
+			if ((okr = kroute_find(prefix.s_addr, prefixlen, prio))
+			    != NULL) {
 				/* just add new multipath routes */
 				if (mpath && rtm->rtm_type == RTM_ADD)
 					goto add;
@@ -1443,13 +1451,6 @@ dispatch_rtmsg(void)
 					goto add;
 				}
 
-				/*
-				 * ospf route overridden by kernel. Preference
-				 * of the route is not checked because this is
-				 * forced -- most probably by a user.
-				 */
-				if (kr->r.flags & F_OSPFD_INSERTED)
-					flags |= F_OSPFD_INSERTED;
 				if (kr->r.flags & F_REDISTRIBUTED)
 					flags |= F_REDISTRIBUTED;
 				kr->r.nexthop.s_addr = nexthop.s_addr;
@@ -1486,6 +1487,7 @@ add:
 				kr->r.nexthop.s_addr = nexthop.s_addr;
 				kr->r.flags = flags;
 				kr->r.ifindex = ifindex;
+				kr->r.priority = prio;
 
 				if ((label = (struct sockaddr_rtlabel *)
 				    rti_info[RTAX_LABEL]) != NULL) {
@@ -1499,8 +1501,8 @@ add:
 			}
 			break;
 		case RTM_DELETE:
-			if ((kr = kroute_find(prefix.s_addr, prefixlen)) ==
-			    NULL)
+			if ((kr = kroute_find(prefix.s_addr, prefixlen, prio))
+			    == NULL)
 				continue;
 			if (!(kr->r.flags & F_KERNEL))
 				continue;
@@ -1512,21 +1514,15 @@ add:
 				    " not found");
 				return (-1);
 			}
-			/*
-			 * last route is getting removed request the
-			 * ospf route from the RDE to insert instead
-			 */
-			if (okr == kr && kr->next == NULL &&
-			    kr->r.flags & F_OSPFD_INSERTED)
-				main_imsg_compose_rde(IMSG_KROUTE_GET, 0,
-				    &kr->r, sizeof(struct kroute));
 			if (kroute_remove(kr) == -1)
 				return (-1);
 			break;
 		case RTM_IFINFO:
 			memcpy(&ifm, next, sizeof(ifm));
-			if_change(ifm.ifm_index, ifm.ifm_flags,
-			    &ifm.ifm_data);
+			sa = (struct sockaddr *)(next + rtm->rtm_hdrlen);
+			get_rtaddrs(ifm.ifm_addrs, sa, rti_info);
+			if_change(ifm.ifm_index, ifm.ifm_flags, &ifm.ifm_data,
+			    (struct sockaddr_dl *)rti_info[RTAX_IFP]);
 			break;
 		case RTM_NEWADDR:
 			ifam = (struct ifa_msghdr *)rtm;
