@@ -1,4 +1,4 @@
-/*	$OpenBSD: lsreq.c,v 1.14 2006/03/13 09:36:06 claudio Exp $ */
+/*	$OpenBSD: lsreq.c,v 1.18 2009/06/06 07:31:26 eric Exp $ */
 
 /*
  * Copyright (c) 2004, 2005 Esben Norby <norby@openbsd.org>
@@ -27,7 +27,7 @@
 #include "log.h"
 #include "ospfe.h"
 
-extern struct imsgbuf		*ibuf_rde;
+extern struct imsgev		*iev_rde;
 
 /* link state request packet handling */
 int
@@ -66,8 +66,8 @@ send_ls_req(struct nbr *nbr)
 
 	/* LSA header(s), keep space for a possible md5 sum */
 	for (le = TAILQ_FIRST(&nbr->ls_req_list); le != NULL &&
-	    buf->wpos + sizeof(struct ls_req_hdr) < buf->max -
-	    MD5_DIGEST_LENGTH; le = nle) {
+	    buf_left(buf) >= sizeof(struct ls_req_hdr) + MD5_DIGEST_LENGTH;
+	    le = nle) {
 		nbr->ls_req = nle = TAILQ_NEXT(le, entry);
 		ls_req_hdr.type = htonl(le->le_lsa->type);
 		ls_req_hdr.ls_id = le->le_lsa->ls_id;
@@ -80,7 +80,7 @@ send_ls_req(struct nbr *nbr)
 	if (auth_gen(buf, nbr->iface))
 		goto fail;
 
-	ret = send_packet(nbr->iface, buf->buf, buf->wpos, &dst);
+	ret = send_packet(nbr->iface, buf, &dst);
 
 	buf_free(buf);
 	return (ret);
@@ -107,7 +107,8 @@ recv_ls_req(struct nbr *nbr, char *buf, u_int16_t len)
 	case NBR_STA_XCHNG:
 	case NBR_STA_LOAD:
 	case NBR_STA_FULL:
-		imsg_compose(ibuf_rde, IMSG_LS_REQ, nbr->peerid, 0, buf, len);
+		imsg_compose_event(iev_rde, IMSG_LS_REQ, nbr->peerid,
+		    0, -1, buf, len);
 		break;
 	default:
 		fatalx("recv_ls_req: unknown neighbor state");
