@@ -1,4 +1,4 @@
-/*	$OpenBSD: ospfd.c,v 1.57 2008/05/12 19:15:02 pyr Exp $ */
+/*	$OpenBSD: ospfd.c,v 1.70 2009/06/06 07:31:26 eric Exp $ */
 
 /*
  * Copyright (c) 2005 Claudio Jeker <claudio@openbsd.org>
@@ -55,8 +55,6 @@ int		check_child(pid_t, const char *);
 void	main_dispatch_ospfe(int, short, void *);
 void	main_dispatch_rde(int, short, void *);
 
-void	ospf_redistribute_default(int);
-
 int	ospf_reload(void);
 int	ospf_sendboth(enum imsg_type, void *, u_int16_t);
 int	merge_interfaces(struct area *, struct area *);
@@ -67,8 +65,8 @@ int	pipe_parent2rde[2];
 int	pipe_ospfe2rde[2];
 
 struct ospfd_conf	*ospfd_conf = NULL;
-struct imsgbuf		*ibuf_ospfe;
-struct imsgbuf		*ibuf_rde;
+struct imsgev		*iev_ospfe;
+struct imsgev		*iev_rde;
 char			*conffile;
 
 pid_t			 ospfe_pid = 0;
@@ -118,7 +116,8 @@ usage(void)
 {
 	extern char *__progname;
 
-	fprintf(stderr, "usage: %s [-cdnv] [-D macro=value] [-f file]\n",
+	fprintf(stderr, "usage: %s [-cdnv] [-D macro=value]"
+	    " [-f file] [-s socket]\n",
 	    __progname);
 	exit(1);
 }
@@ -133,13 +132,15 @@ main(int argc, char *argv[])
 	int			 ipforwarding;
 	int			 mib[4];
 	size_t			 len;
+	char			*sockname;
 
 	conffile = CONF_FILE;
 	ospfd_process = PROC_MAIN;
+	sockname = OSPFD_SOCKET;
 
 	log_init(1);	/* log to stderr until daemonized */
 
-	while ((ch = getopt(argc, argv, "cdD:f:nv")) != -1) {
+	while ((ch = getopt(argc, argv, "cdD:f:ns:v")) != -1) {
 		switch (ch) {
 		case 'c':
 			opts |= OSPFD_OPT_FORCE_DEMOTE;
@@ -158,6 +159,9 @@ main(int argc, char *argv[])
 		case 'n':
 			opts |= OSPFD_OPT_NOACTION;
 			break;
+		case 's':
+			sockname = optarg;
+			break;
 		case 'v':
 			if (opts & OSPFD_OPT_VERBOSE)
 				opts |= OSPFD_OPT_VERBOSE2;
@@ -194,14 +198,18 @@ main(int argc, char *argv[])
 	kif_init();
 
 	/* parse config file */
-	if ((ospfd_conf = parse_config(conffile, opts)) == NULL )
+	if ((ospfd_conf = parse_config(conffile, opts)) == NULL) {
+		kr_shutdown();
 		exit(1);
+	}
+	ospfd_conf->csock = sockname;
 
 	if (ospfd_conf->opts & OSPFD_OPT_NOACTION) {
 		if (ospfd_conf->opts & OSPFD_OPT_VERBOSE)
 			print_config(ospfd_conf);
 		else
 			fprintf(stderr, "configuration OK\n");
+		kr_shutdown();
 		exit(0);
 	}
 
@@ -262,24 +270,27 @@ main(int argc, char *argv[])
 	close(pipe_ospfe2rde[0]);
 	close(pipe_ospfe2rde[1]);
 
-	if ((ibuf_ospfe = malloc(sizeof(struct imsgbuf))) == NULL ||
-	    (ibuf_rde = malloc(sizeof(struct imsgbuf))) == NULL)
+	if ((iev_ospfe = malloc(sizeof(struct imsgev))) == NULL ||
+	    (iev_rde = malloc(sizeof(struct imsgev))) == NULL)
 		fatal(NULL);
-	imsg_init(ibuf_ospfe, pipe_parent2ospfe[0], main_dispatch_ospfe);
-	imsg_init(ibuf_rde, pipe_parent2rde[0], main_dispatch_rde);
+	imsg_init(&iev_ospfe->ibuf, pipe_parent2ospfe[0]);
+	iev_ospfe->handler = main_dispatch_ospfe;
+	imsg_init(&iev_rde->ibuf, pipe_parent2rde[0]);
+	iev_rde->handler = main_dispatch_rde;
 
 	/* setup event handler */
-	ibuf_ospfe->events = EV_READ;
-	event_set(&ibuf_ospfe->ev, ibuf_ospfe->fd, ibuf_ospfe->events,
-	    ibuf_ospfe->handler, ibuf_ospfe);
-	event_add(&ibuf_ospfe->ev, NULL);
+	iev_ospfe->events = EV_READ;
+	event_set(&iev_ospfe->ev, iev_ospfe->ibuf.fd, iev_ospfe->events,
+	    iev_ospfe->handler, iev_ospfe);
+	event_add(&iev_ospfe->ev, NULL);
 
-	ibuf_rde->events = EV_READ;
-	event_set(&ibuf_rde->ev, ibuf_rde->fd, ibuf_rde->events,
-	    ibuf_rde->handler, ibuf_rde);
-	event_add(&ibuf_rde->ev, NULL);
+	iev_rde->events = EV_READ;
+	event_set(&iev_rde->ev, iev_rde->ibuf.fd, iev_rde->events,
+	    iev_rde->handler, iev_rde);
+	event_add(&iev_rde->ev, NULL);
 
-	if (kr_init(!(ospfd_conf->flags & OSPFD_FLAG_NO_FIB_UPDATE)) == -1)
+	if (kr_init(!(ospfd_conf->flags & OSPFD_FLAG_NO_FIB_UPDATE),
+	    ospfd_conf->rdomain) == -1)
 		fatalx("kr_init failed");
 
 	/* remove unneded stuff from config */
@@ -288,9 +299,6 @@ main(int argc, char *argv[])
 		area_del(a);
 	}
 
-	/* redistribute default */
-	ospf_redistribute_default(IMSG_NETWORK_ADD);
-
 	event_dispatch();
 
 	ospfd_shutdown();
@@ -309,7 +317,7 @@ ospfd_shutdown(void)
 	if (rde_pid)
 		kill(rde_pid, SIGTERM);
 
-	control_cleanup();
+	control_cleanup(ospfd_conf->csock);
 	kr_shutdown();
 	carp_demote_shutdown();
 
@@ -319,10 +327,10 @@ ospfd_shutdown(void)
 			fatal("wait");
 	} while (pid != -1 || (pid == -1 && errno == EINTR));
 
-	msgbuf_clear(&ibuf_ospfe->w);
-	free(ibuf_ospfe);
-	msgbuf_clear(&ibuf_rde->w);
-	free(ibuf_rde);
+	msgbuf_clear(&iev_ospfe->ibuf.w);
+	free(iev_ospfe);
+	msgbuf_clear(&iev_rde->ibuf.w);
+	free(iev_rde);
 	free(ospfd_conf);
 
 	log_info("terminating");
@@ -354,26 +362,24 @@ check_child(pid_t pid, const char *pname)
 void
 main_dispatch_ospfe(int fd, short event, void *bula)
 {
-	struct imsgbuf		*ibuf = bula;
+	struct imsgev		*iev = bula;
+	struct imsgbuf		*ibuf;
 	struct imsg		 imsg;
 	struct demote_msg	 dmsg;
 	ssize_t			 n;
 	int			 shut = 0;
 
-	switch (event) {
-	case EV_READ:
+	ibuf = &iev->ibuf;
+
+	if (event & EV_READ) {
 		if ((n = imsg_read(ibuf)) == -1)
 			fatal("imsg_read error");
 		if (n == 0)	/* connection closed */
 			shut = 1;
-		break;
-	case EV_WRITE:
+	}
+	if (event & EV_WRITE) {
 		if (msgbuf_write(&ibuf->w) == -1)
 			fatal("msgbuf_write");
-		imsg_event_add(ibuf);
-		return;
-	default:
-		fatalx("unknown event");
 	}
 
 	for (;;) {
@@ -422,10 +428,10 @@ main_dispatch_ospfe(int fd, short event, void *bula)
 		imsg_free(&imsg);
 	}
 	if (!shut)
-		imsg_event_add(ibuf);
+		imsg_event_add(iev);
 	else {
 		/* this pipe is dead, so remove the event handler */
-		event_del(&ibuf->ev);
+		event_del(&iev->ev);
 		event_loopexit(NULL);
 	}
 }
@@ -434,25 +440,23 @@ main_dispatch_ospfe(int fd, short event, void *bula)
 void
 main_dispatch_rde(int fd, short event, void *bula)
 {
-	struct imsgbuf  *ibuf = bula;
+	struct imsgev	*iev = bula;
+	struct imsgbuf  *ibuf;
 	struct imsg	 imsg;
 	ssize_t		 n;
 	int		 count, shut = 0;
 
-	switch (event) {
-	case EV_READ:
+	ibuf = &iev->ibuf;
+
+	if (event & EV_READ) {
 		if ((n = imsg_read(ibuf)) == -1)
 			fatal("imsg_read error");
 		if (n == 0)	/* connection closed */
 			shut = 1;
-		break;
-	case EV_WRITE:
+	}
+	if (event & EV_WRITE) {
 		if (msgbuf_write(&ibuf->w) == -1)
 			fatal("msgbuf_write");
-		imsg_event_add(ibuf);
-		return;
-	default:
-		fatalx("unknown event");
 	}
 
 	for (;;) {
@@ -483,10 +487,10 @@ main_dispatch_rde(int fd, short event, void *bula)
 		imsg_free(&imsg);
 	}
 	if (!shut)
-		imsg_event_add(ibuf);
+		imsg_event_add(iev);
 	else {
 		/* this pipe is dead, so remove the event handler */
-		event_del(&ibuf->ev);
+		event_del(&iev->ev);
 		event_loopexit(NULL);
 	}
 }
@@ -494,36 +498,48 @@ main_dispatch_rde(int fd, short event, void *bula)
 void
 main_imsg_compose_ospfe(int type, pid_t pid, void *data, u_int16_t datalen)
 {
-	imsg_compose(ibuf_ospfe, type, 0, pid, data, datalen);
+	imsg_compose_event(iev_ospfe, type, 0, pid, -1, data, datalen);
 }
 
 void
 main_imsg_compose_rde(int type, pid_t pid, void *data, u_int16_t datalen)
 {
-	imsg_compose(ibuf_rde, type, 0, pid, data, datalen);
+	imsg_compose_event(iev_rde, type, 0, pid, -1, data, datalen);
 }
 
-/* this needs to be added here so that ospfctl can be used without libevent */
 void
-imsg_event_add(struct imsgbuf *ibuf)
+imsg_event_add(struct imsgev *iev)
 {
-	ibuf->events = EV_READ;
-	if (ibuf->w.queued)
-		ibuf->events |= EV_WRITE;
+	iev->events = EV_READ;
+	if (iev->ibuf.w.queued)
+		iev->events |= EV_WRITE;
 
-	event_del(&ibuf->ev);
-	event_set(&ibuf->ev, ibuf->fd, ibuf->events, ibuf->handler, ibuf);
-	event_add(&ibuf->ev, NULL);
+	event_del(&iev->ev);
+	event_set(&iev->ev, iev->ibuf.fd, iev->events, iev->handler, iev);
+	event_add(&iev->ev, NULL);
+}
+
+int
+imsg_compose_event(struct imsgev *iev, u_int16_t type, u_int32_t peerid,
+    pid_t pid, int fd, void *data, u_int16_t datalen)
+{
+	int	ret;
+
+	if ((ret = imsg_compose(&iev->ibuf, type, peerid,
+	    pid, fd, data, datalen)) != -1)
+		imsg_event_add(iev);
+	return (ret);
 }
 
 int
 ospf_redistribute(struct kroute *kr, u_int32_t *metric)
 {
 	struct redistribute	*r;
+	u_int8_t		 is_default = 0;
 
-	/* only allow 0.0.0.0/0 via REDISTRIBUTE_DEFAULT */
+	/* only allow 0.0.0.0/0 via REDIST_DEFAULT */
 	if (kr->prefix.s_addr == INADDR_ANY && kr->prefixlen == 0)
-		return (0);
+		is_default = 1;
 
 	SIMPLEQ_FOREACH(r, &ospfd_conf->redist_list, entry) {
 		switch (r->type & ~REDIST_NO) {
@@ -539,6 +555,8 @@ ospf_redistribute(struct kroute *kr, u_int32_t *metric)
 			 * so that link local addresses can be redistributed
 			 * via a rtlabel.
 			 */
+			if (is_default)
+				continue;
 			if (kr->flags & F_DYNAMIC)
 				continue;
 			if (kr->flags & F_STATIC) {
@@ -547,6 +565,8 @@ ospf_redistribute(struct kroute *kr, u_int32_t *metric)
 			}
 			break;
 		case REDIST_CONNECTED:
+			if (is_default)
+				continue;
 			if (kr->flags & F_DYNAMIC)
 				continue;
 			if (kr->flags & F_CONNECTED) {
@@ -557,6 +577,16 @@ ospf_redistribute(struct kroute *kr, u_int32_t *metric)
 		case REDIST_ADDR:
 			if (kr->flags & F_DYNAMIC)
 				continue;
+
+			if (r->addr.s_addr == INADDR_ANY &&
+			    r->mask.s_addr == INADDR_ANY) {
+				if (is_default) {
+					*metric = r->metric;
+					return (r->type & REDIST_NO? 0 : 1);
+				} else
+					return (0);
+			}
+
 			if ((kr->prefix.s_addr & r->mask.s_addr) ==
 			    (r->addr.s_addr & r->mask.s_addr) &&
 			    kr->prefixlen >= mask2prefixlen(r->mask.s_addr)) {
@@ -564,31 +594,25 @@ ospf_redistribute(struct kroute *kr, u_int32_t *metric)
 				return (r->type & REDIST_NO ? 0 : 1);
 			}
 			break;
+		case REDIST_DEFAULT:
+			if (is_default) {
+				*metric = r->metric;
+				return (r->type & REDIST_NO ? 0 : 1);
+			}
+			break;
 		}
 	}
 
 	return (0);
 }
 
-void
-ospf_redistribute_default(int type)
-{
-	struct rroute	rr;
-
-	if (!(ospfd_conf->redistribute & REDISTRIBUTE_DEFAULT))
-		return;
-
-	bzero(&rr, sizeof(rr));
-	rr.metric = ospfd_conf->defaultmetric;
-	main_imsg_compose_rde(type, 0, &rr, sizeof(struct rroute));
-}
-
 int
 ospf_reload(void)
 {
 	struct area		*area;
 	struct iface		*iface;
 	struct ospfd_conf	*xconf;
+	struct redistribute	*r;
 
 	if ((xconf = parse_config(conffile, ospfd_conf->opts)) == NULL)
 		return (-1);
@@ -602,13 +626,17 @@ ospf_reload(void)
 		if (ospf_sendboth(IMSG_RECONF_AREA, area, sizeof(*area)) == -1)
 			return (-1);
 
+		SIMPLEQ_FOREACH(r, &area->redist_list, entry) {
+			main_imsg_compose_rde(IMSG_RECONF_REDIST, 0, r,
+			    sizeof(*r));
+		}
 		LIST_FOREACH(iface, &area->iface_list, entry) {
 			if (ospf_sendboth(IMSG_RECONF_IFACE, iface,
 			    sizeof(*iface)) == -1)
 				return (-1);
 			if (iface->auth_type == AUTH_CRYPT)
 				if (md_list_send(&iface->auth_md_list,
-				    ibuf_ospfe) == -1)
+				    iev_ospfe) == -1)
 					return (-1);
 		}
 	}
@@ -625,9 +653,9 @@ ospf_reload(void)
 int
 ospf_sendboth(enum imsg_type type, void *buf, u_int16_t len)
 {
-	if (imsg_compose(ibuf_ospfe, type, 0, 0, buf, len) == -1)
+	if (imsg_compose_event(iev_ospfe, type, 0, 0, -1, buf, len) == -1)
 		return (-1);
-	if (imsg_compose(ibuf_rde, type, 0, 0, buf, len) == -1)
+	if (imsg_compose_event(iev_rde, type, 0, 0, -1, buf, len) == -1)
 		return (-1);
 	return (0);
 }
@@ -644,10 +672,9 @@ merge_config(struct ospfd_conf *conf, struct ospfd_conf *xconf)
 	conf->flags = xconf->flags;
 	conf->spf_delay = xconf->spf_delay;
 	conf->spf_hold_time = xconf->spf_hold_time;
-	if ((conf->redistribute & REDISTRIBUTE_ON) !=
-	    (xconf->redistribute & REDISTRIBUTE_ON))
+	if (SIMPLEQ_EMPTY(&conf->redist_list) !=
+	    SIMPLEQ_EMPTY(&xconf->redist_list))
 		rchange = 1;
-	conf->redistribute = xconf->redistribute;
 	conf->rfc1583compat = xconf->rfc1583compat;
 
 	if (ospfd_process == PROC_MAIN) {
@@ -701,6 +728,20 @@ merge_config(struct ospfd_conf *conf, struct ospfd_conf *xconf)
 		 * stub is not yet used but switching between stub and normal
 		 * will be another painful job.
 		 */
+		if (a->stub != xa->stub && ospfd_process == PROC_OSPF_ENGINE)
+			a->dirty = 1; /* force rtr LSA update */
+		if (xa->stub && ospfd_process == PROC_RDE_ENGINE) {
+			while ((r = SIMPLEQ_FIRST(&a->redist_list)) != NULL) {
+				SIMPLEQ_REMOVE_HEAD(&a->redist_list, entry);
+				free(r);
+			}
+			
+			while ((r = SIMPLEQ_FIRST(&xa->redist_list)) != NULL) {
+				SIMPLEQ_REMOVE_HEAD(&xa->redist_list, entry);
+				SIMPLEQ_INSERT_TAIL(&a->redist_list, r, entry);
+			}
+		}
+
 		a->stub = xa->stub;
 		a->stub_default_cost = xa->stub_default_cost;
 		if (ospfd_process == PROC_RDE_ENGINE)
@@ -731,6 +772,14 @@ merge_config(struct ospfd_conf *conf, struct ospfd_conf *xconf)
 			}
 		}
 	}
+	if (ospfd_process == PROC_RDE_ENGINE) {
+		LIST_FOREACH(a, &conf->area_list, entry) {
+			if (a->dirty) {
+				start_spf_timer();
+				break;
+			}
+		}
+	}
 
 done:
 	while ((a = LIST_FIRST(&xconf->area_list)) != NULL) {
@@ -754,7 +803,7 @@ merge_interfaces(struct area *a, struct area *xa)
 	for (i = LIST_FIRST(&a->iface_list); i != NULL; i = ni) {
 		ni = LIST_NEXT(i, entry);
 		if (iface_lookup(xa, i) == NULL) {
-			log_debug("merge_config: proc %d area %s removing "
+			log_debug("merge_interfaces: proc %d area %s removing "
 			    "interface %s", ospfd_process, inet_ntoa(a->id),
 			    i->name);
 			if (ospfd_process == PROC_OSPF_ENGINE)
@@ -768,7 +817,7 @@ merge_interfaces(struct area *a, struct area *xa)
 		ni = LIST_NEXT(xi, entry);
 		if ((i = iface_lookup(a, xi)) == NULL) {
 			/* new interface but delay initialisation */
-			log_debug("merge_config: proc %d area %s adding "
+			log_debug("merge_interfaces: proc %d area %s adding "
 			    "interface %s", ospfd_process, inet_ntoa(a->id),
 			    xi->name);
 			LIST_REMOVE(xi, entry);
@@ -778,8 +827,8 @@ merge_interfaces(struct area *a, struct area *xa)
 				xi->state = IF_STA_NEW;
 			continue;
 		}
-		log_debug("merge_config: proc %d area %s merging interface %s",
-		    ospfd_process, inet_ntoa(a->id), i->name);
+		log_debug("merge_interfaces: proc %d area %s merging "
+		    "interface %s", ospfd_process, inet_ntoa(a->id), i->name);
 		i->addr = xi->addr;
 		i->dst = xi->dst;
 		i->mask = xi->mask;
