@@ -1,4 +1,4 @@
-/*	$OpenBSD: ospfd.h,v 1.71 2007/10/11 12:19:31 claudio Exp $ */
+/*	$OpenBSD: ospfd.h,v 1.83 2009/06/06 18:31:42 pyr Exp $ */
 
 /*
  * Copyright (c) 2004 Esben Norby <norby@openbsd.org>
@@ -29,6 +29,7 @@
 #include <netinet/in.h>
 #include <event.h>
 
+#include <imsg.h>
 #include "ospf.h"
 
 #define CONF_FILE		"/etc/ospfd.conf"
@@ -51,50 +52,20 @@
 
 #define	F_OSPFD_INSERTED	0x0001
 #define	F_KERNEL		0x0002
-#define	F_BGPD_INSERTED		0x0004
-#define	F_CONNECTED		0x0008
-#define	F_DOWN			0x0010
-#define	F_STATIC		0x0020
-#define	F_DYNAMIC		0x0040
+#define	F_CONNECTED		0x0004
+#define	F_STATIC		0x0008
+#define	F_DYNAMIC		0x0010
+#define	F_DOWN			0x0020
+#define	F_REJECT		0x0040
+#define	F_BLACKHOLE		0x0080
 #define	F_REDISTRIBUTED		0x0100
 
-#define REDISTRIBUTE_ON		0x01
-#define REDISTRIBUTE_DEFAULT	0x02
-
-/* buffer */
-struct buf {
-	TAILQ_ENTRY(buf)	 entry;
-	u_char			*buf;
-	size_t			 size;
-	size_t			 max;
-	size_t			 wpos;
-	size_t			 rpos;
-};
-
-struct msgbuf {
-	TAILQ_HEAD(, buf)	 bufs;
-	u_int32_t		 queued;
-	int			 fd;
-};
-
-#define	IMSG_HEADER_SIZE	sizeof(struct imsg_hdr)
-#define	MAX_IMSGSIZE		8192
-
-struct buf_read {
-	u_char			 buf[READ_BUF_SIZE];
-	u_char			*rptr;
-	size_t			 wpos;
-};
-
-struct imsgbuf {
-	TAILQ_HEAD(, imsg_fd)	fds;
-	struct buf_read		r;
-	struct msgbuf		w;
-	struct event		ev;
+struct imsgev {
+	struct imsgbuf		 ibuf;
 	void			(*handler)(int, short, void *);
-	int			fd;
-	pid_t			pid;
-	short			events;
+	struct event		 ev;
+	void			*data;
+	short			 events;
 };
 
 enum imsg_type {
@@ -121,7 +92,6 @@ enum imsg_type {
 	IMSG_CTL_END,
 	IMSG_KROUTE_CHANGE,
 	IMSG_KROUTE_DELETE,
-	IMSG_KROUTE_GET,
 	IMSG_IFINFO,
 	IMSG_NEIGHBOR_UP,
 	IMSG_NEIGHBOR_DOWN,
@@ -130,6 +100,7 @@ enum imsg_type {
 	IMSG_NETWORK_DEL,
 	IMSG_DD,
 	IMSG_DD_END,
+	IMSG_DD_BADLSA,
 	IMSG_DB_SNAPSHOT,
 	IMSG_DB_END,
 	IMSG_LS_REQ,
@@ -144,23 +115,28 @@ enum imsg_type {
 	IMSG_RECONF_AREA,
 	IMSG_RECONF_IFACE,
 	IMSG_RECONF_AUTHMD,
+	IMSG_RECONF_REDIST,
 	IMSG_RECONF_END,
 	IMSG_DEMOTE
 };
 
-struct imsg_hdr {
-	enum imsg_type	type;
-	u_int16_t	len;
-	u_int32_t	peerid;
-	pid_t		pid;
-};
+#define	REDIST_CONNECTED	0x01
+#define	REDIST_STATIC		0x02
+#define	REDIST_LABEL		0x04
+#define	REDIST_ADDR		0x08
+#define	REDIST_NO		0x10
+#define	REDIST_DEFAULT		0x20
 
-struct imsg {
-	struct imsg_hdr	 hdr;
-	void		*data;
+struct redistribute {
+	SIMPLEQ_ENTRY(redistribute)	entry;
+	struct in_addr			addr;
+	struct in_addr			mask;
+	u_int32_t			metric;
+	u_int16_t			label;
+	u_int16_t			type;
 };
+SIMPLEQ_HEAD(redist_list, redistribute);
 
-/* area */
 struct vertex;
 struct rde_nbr;
 RB_HEAD(lsa_tree, vertex);
@@ -172,6 +148,7 @@ struct area {
 
 	LIST_HEAD(, iface)	 iface_list;
 	LIST_HEAD(, rde_nbr)	 nbr_list;
+	struct redist_list	 redist_list;
 /*	list			 addr_range_list; */
 	char			 demote_group[IFNAMSIZ];
 	u_int32_t		 stub_default_cost;
@@ -372,29 +349,13 @@ enum {
 	PROC_RDE_ENGINE
 } ospfd_process;
 
-#define	REDIST_CONNECTED	0x01
-#define	REDIST_STATIC		0x02
-#define	REDIST_LABEL		0x04
-#define	REDIST_ADDR		0x08
-#define	REDIST_NO		0x10
-
-struct redistribute {
-	SIMPLEQ_ENTRY(redistribute)	entry;
-	struct in_addr			addr;
-	struct in_addr			mask;
-	u_int32_t			metric;
-	u_int16_t			label;
-	u_int16_t			type;
-};
-
 struct ospfd_conf {
 	struct event		ev;
 	struct in_addr		rtr_id;
 	LIST_HEAD(, area)	area_list;
 	LIST_HEAD(, vertex)	cand_list;
-	SIMPLEQ_HEAD(, redistribute) redist_list;
+	struct redist_list	redist_list;
 
-	u_int32_t		defaultmetric;
 	u_int32_t		opts;
 #define OSPFD_OPT_VERBOSE	0x00000001
 #define OSPFD_OPT_VERBOSE2	0x00000002
@@ -410,17 +371,20 @@ struct ospfd_conf {
 	u_int8_t		rfc1583compat;
 	u_int8_t		border;
 	u_int8_t		redistribute;
+	u_int			rdomain;
+	char			*csock;
 };
 
 /* kroute */
 struct kroute {
 	struct in_addr	prefix;
 	struct in_addr	nexthop;
+	u_int32_t	ext_tag;
 	u_int16_t	flags;
 	u_int16_t	rtlabel;
-	u_int32_t	ext_tag;
 	u_short		ifindex;
 	u_int8_t	prefixlen;
+	u_int8_t	priority;
 };
 
 struct rroute {
@@ -556,18 +520,6 @@ void		 area_track(struct area *, int);
 int		 area_border_router(struct ospfd_conf *);
 u_int8_t	 area_ospf_options(struct area *);
 
-/* buffer.c */
-struct buf	*buf_open(size_t);
-struct buf	*buf_dynamic(size_t, size_t);
-int		 buf_add(struct buf *, void *, size_t);
-void		*buf_reserve(struct buf *, size_t);
-void		*buf_seek(struct buf *, size_t, size_t);
-int		 buf_close(struct msgbuf *, struct buf *);
-void		 buf_free(struct buf *);
-void		 msgbuf_init(struct msgbuf *);
-void		 msgbuf_clear(struct msgbuf *);
-int		 msgbuf_write(struct msgbuf *);
-
 /* carp.c */
 int		 carp_demote_init(char *, int);
 void		 carp_demote_shutdown(void);
@@ -578,19 +530,6 @@ int		 carp_demote_set(char *, int);
 struct ospfd_conf	*parse_config(char *, int);
 int			 cmdline_symset(char *);
 
-/* imsg.c */
-void	 imsg_init(struct imsgbuf *, int, void (*)(int, short, void *));
-ssize_t	 imsg_read(struct imsgbuf *);
-ssize_t	 imsg_get(struct imsgbuf *, struct imsg *);
-int	 imsg_compose(struct imsgbuf *, enum imsg_type, u_int32_t, pid_t,
-	    void *, u_int16_t);
-struct buf	*imsg_create(struct imsgbuf *, enum imsg_type, u_int32_t, pid_t,
-		    u_int16_t);
-int	 imsg_add(struct buf *, void *, u_int16_t);
-int	 imsg_close(struct imsgbuf *, struct buf *);
-void	 imsg_free(struct imsg *);
-void	 imsg_event_add(struct imsgbuf *); /* needs to be provided externally */
-
 /* in_cksum.c */
 u_int16_t	 in_cksum(void *, size_t);
 
@@ -599,7 +538,7 @@ u_int16_t	 iso_cksum(void *, u_int16_t, u_int16_t);
 
 /* kroute.c */
 int		 kif_init(void);
-int		 kr_init(int);
+int		 kr_init(int, u_int);
 int		 kr_change(struct kroute *, int);
 int		 kr_delete(struct kroute *);
 void		 kr_shutdown(void);
@@ -635,6 +574,9 @@ void	main_imsg_compose_ospfe(int, pid_t, void *, u_int16_t);
 void	main_imsg_compose_rde(int, pid_t, void *, u_int16_t);
 int	ospf_redistribute(struct kroute *, u_int32_t *);
 void	merge_config(struct ospfd_conf *, struct ospfd_conf *);
+void	imsg_event_add(struct imsgev *);
+int	imsg_compose_event(struct imsgev *, u_int16_t, u_int32_t,
+	    pid_t, int, void *, u_int16_t);
 
 /* printconf.c */
 void	print_config(struct ospfd_conf *);
