@@ -1,4 +1,4 @@
-/*	$OpenBSD: rde.c,v 1.73 2008/02/11 12:37:37 norby Exp $ */
+/*	$OpenBSD: rde.c,v 1.81 2009/06/06 07:31:26 eric Exp $ */
 
 /*
  * Copyright (c) 2004, 2005 Claudio Jeker <claudio@openbsd.org>
@@ -63,8 +63,8 @@ struct lsa	*orig_asext_lsa(struct rroute *, u_int16_t);
 struct lsa	*orig_sum_lsa(struct rt_node *, struct area *, u_int8_t, int);
 
 struct ospfd_conf	*rdeconf = NULL, *nconf = NULL;
-struct imsgbuf		*ibuf_ospfe;
-struct imsgbuf		*ibuf_main;
+struct imsgev		*iev_ospfe;
+struct imsgev		*iev_main;
 struct rde_nbr		*nbrself;
 struct lsa_tree		 asext_tree;
 
@@ -145,22 +145,24 @@ rde(struct ospfd_conf *xconf, int pipe_parent2rde[2], int pipe_ospfe2rde[2],
 	close(pipe_parent2ospfe[0]);
 	close(pipe_parent2ospfe[1]);
 
-	if ((ibuf_ospfe = malloc(sizeof(struct imsgbuf))) == NULL ||
-	    (ibuf_main = malloc(sizeof(struct imsgbuf))) == NULL)
+	if ((iev_ospfe = malloc(sizeof(struct imsgev))) == NULL ||
+	    (iev_main = malloc(sizeof(struct imsgev))) == NULL)
 		fatal(NULL);
-	imsg_init(ibuf_ospfe, pipe_ospfe2rde[1], rde_dispatch_imsg);
-	imsg_init(ibuf_main, pipe_parent2rde[1], rde_dispatch_parent);
+	imsg_init(&iev_ospfe->ibuf, pipe_ospfe2rde[1]);
+	iev_ospfe->handler = rde_dispatch_imsg;
+	imsg_init(&iev_main->ibuf, pipe_parent2rde[1]);
+	iev_main->handler = rde_dispatch_parent;
 
 	/* setup event handler */
-	ibuf_ospfe->events = EV_READ;
-	event_set(&ibuf_ospfe->ev, ibuf_ospfe->fd, ibuf_ospfe->events,
-	    ibuf_ospfe->handler, ibuf_ospfe);
-	event_add(&ibuf_ospfe->ev, NULL);
+	iev_ospfe->events = EV_READ;
+	event_set(&iev_ospfe->ev, iev_ospfe->ibuf.fd, iev_ospfe->events,
+	    iev_ospfe->handler, iev_ospfe);
+	event_add(&iev_ospfe->ev, NULL);
 
-	ibuf_main->events = EV_READ;
-	event_set(&ibuf_main->ev, ibuf_main->fd, ibuf_main->events,
-	    ibuf_main->handler, ibuf_main);
-	event_add(&ibuf_main->ev, NULL);
+	iev_main->events = EV_READ;
+	event_set(&iev_main->ev, iev_main->ibuf.fd, iev_main->events,
+	    iev_main->handler, iev_main);
+	event_add(&iev_main->ev, NULL);
 
 	evtimer_set(&rdeconf->ev, spf_timer, rdeconf);
 	cand_list_init();
@@ -202,10 +204,10 @@ rde_shutdown(void)
 	}
 	rde_nbr_free();
 
-	msgbuf_clear(&ibuf_ospfe->w);
-	free(ibuf_ospfe);
-	msgbuf_clear(&ibuf_main->w);
-	free(ibuf_main);
+	msgbuf_clear(&iev_ospfe->ibuf.w);
+	free(iev_ospfe);
+	msgbuf_clear(&iev_main->ibuf.w);
+	free(iev_main);
 	free(rdeconf);
 
 	log_info("route decision engine exiting");
@@ -216,14 +218,15 @@ int
 rde_imsg_compose_ospfe(int type, u_int32_t peerid, pid_t pid, void *data,
     u_int16_t datalen)
 {
-	return (imsg_compose(ibuf_ospfe, type, peerid, pid, data, datalen));
+	return (imsg_compose_event(iev_ospfe, type, peerid, pid, -1, data, datalen));
 }
 
 /* ARGSUSED */
 void
 rde_dispatch_imsg(int fd, short event, void *bula)
 {
-	struct imsgbuf		*ibuf = bula;
+	struct imsgev		*iev = bula;
+	struct imsgbuf		*ibuf;
 	struct imsg		 imsg;
 	struct in_addr		 aid;
 	struct ls_req_hdr	 req_hdr;
@@ -236,23 +239,20 @@ rde_dispatch_imsg(int fd, short event, void *bula)
 	char			*buf;
 	ssize_t			 n;
 	time_t			 now;
-	int			 r, state, self, shut = 0;
+	int			 r, state, self, error, shut = 0;
 	u_int16_t		 l;
 
-	switch (event) {
-	case EV_READ:
+	ibuf = &iev->ibuf;
+
+	if (event & EV_READ) {
 		if ((n = imsg_read(ibuf)) == -1)
 			fatal("imsg_read error");
 		if (n == 0)	/* connection closed */
 			shut = 1;
-		break;
-	case EV_WRITE:
+	}
+	if (event & EV_WRITE) {
 		if (msgbuf_write(&ibuf->w) == -1)
 			fatal("msgbuf_write");
-		imsg_event_add(ibuf);
-		return;
-	default:
-		fatalx("unknown event");
 	}
 
 	clock_gettime(CLOCK_MONOTONIC, &tp);
@@ -270,10 +270,9 @@ rde_dispatch_imsg(int fd, short event, void *bula)
 				fatalx("invalid size of OE request");
 			memcpy(&rn, imsg.data, sizeof(rn));
 
-			if (rde_nbr_find(imsg.hdr.peerid))
+			if (rde_nbr_new(imsg.hdr.peerid, &rn) == NULL)
 				fatalx("rde_dispatch_imsg: "
 				    "neighbor already exists");
-			rde_nbr_new(imsg.hdr.peerid, &rn);
 			break;
 		case IMSG_NEIGHBOR_DOWN:
 			rde_nbr_del(rde_nbr_find(imsg.hdr.peerid));
@@ -302,8 +301,8 @@ rde_dispatch_imsg(int fd, short event, void *bula)
 
 			lsa_snap(nbr->area, imsg.hdr.peerid);
 
-			imsg_compose(ibuf_ospfe, IMSG_DB_END, imsg.hdr.peerid,
-			    0, NULL, 0);
+			imsg_compose_event(iev_ospfe, IMSG_DB_END, imsg.hdr.peerid,
+			    0, -1, NULL, 0);
 			break;
 		case IMSG_DD:
 			nbr = rde_nbr_find(imsg.hdr.peerid);
@@ -311,11 +310,17 @@ rde_dispatch_imsg(int fd, short event, void *bula)
 				break;
 
 			buf = imsg.data;
+			error = 0;
 			for (l = imsg.hdr.len - IMSG_HEADER_SIZE;
 			    l >= sizeof(lsa_hdr); l -= sizeof(lsa_hdr)) {
 				memcpy(&lsa_hdr, buf, sizeof(lsa_hdr));
 				buf += sizeof(lsa_hdr);
 
+				if (lsa_hdr.type == LSA_TYPE_EXTERNAL &&
+				    nbr->area->stub) {
+					error = 1;
+					break;
+				}
 				v = lsa_find(nbr->area, lsa_hdr.type,
 				    lsa_hdr.ls_id, lsa_hdr.adv_rtr);
 				if (v == NULL)
@@ -329,18 +334,22 @@ rde_dispatch_imsg(int fd, short event, void *bula)
 					 * newer or missing
 					 */
 					rde_req_list_add(nbr, &lsa_hdr);
-					imsg_compose(ibuf_ospfe, IMSG_DD,
-					    imsg.hdr.peerid, 0, &lsa_hdr,
+					imsg_compose_event(iev_ospfe, IMSG_DD,
+					    imsg.hdr.peerid, 0, -1, &lsa_hdr,
 					    sizeof(lsa_hdr));
 				}
 			}
-			if (l != 0)
+			if (l != 0 && !error)
 				log_warnx("rde_dispatch_imsg: peerid %lu, "
 				    "trailing garbage in Database Description "
 				    "packet", imsg.hdr.peerid);
 
-			imsg_compose(ibuf_ospfe, IMSG_DD_END, imsg.hdr.peerid,
-			    0, NULL, 0);
+			if (!error)
+				imsg_compose_event(iev_ospfe, IMSG_DD_END,
+				    imsg.hdr.peerid, 0, -1, NULL, 0);
+			else
+				imsg_compose_event(iev_ospfe, IMSG_DD_BADLSA,
+				    imsg.hdr.peerid, 0, -1, NULL, 0);
 			break;
 		case IMSG_LS_REQ:
 			nbr = rde_nbr_find(imsg.hdr.peerid);
@@ -356,12 +365,12 @@ rde_dispatch_imsg(int fd, short event, void *bula)
 				if ((v = lsa_find(nbr->area,
 				    ntohl(req_hdr.type), req_hdr.ls_id,
 				    req_hdr.adv_rtr)) == NULL) {
-					imsg_compose(ibuf_ospfe, IMSG_LS_BADREQ,
-					    imsg.hdr.peerid, 0, NULL, 0);
+					imsg_compose_event(iev_ospfe, IMSG_LS_BADREQ,
+					    imsg.hdr.peerid, 0, -1, NULL, 0);
 					continue;
 				}
-				imsg_compose(ibuf_ospfe, IMSG_LS_UPD,
-				    imsg.hdr.peerid, 0, v->lsa,
+				imsg_compose_event(iev_ospfe, IMSG_LS_UPD,
+				    imsg.hdr.peerid, 0, -1, v->lsa,
 				    ntohs(v->lsa->hdr.len));
 			}
 			if (l != 0)
@@ -415,14 +424,14 @@ rde_dispatch_imsg(int fd, short event, void *bula)
 						break;
 
 				/* flood and perhaps ack LSA */
-				imsg_compose(ibuf_ospfe, IMSG_LS_FLOOD,
-				    imsg.hdr.peerid, 0, lsa,
+				imsg_compose_event(iev_ospfe, IMSG_LS_FLOOD,
+				    imsg.hdr.peerid, 0, -1, lsa,
 				    ntohs(lsa->hdr.len));
 
 				/* reflood self originated LSA */
 				if (self && v)
-					imsg_compose(ibuf_ospfe, IMSG_LS_FLOOD,
-					    v->peerid, 0, v->lsa,
+					imsg_compose_event(iev_ospfe, IMSG_LS_FLOOD,
+					    v->peerid, 0, -1, v->lsa,
 					    ntohs(v->lsa->hdr.len));
 				/* lsa not added so free it */
 				if (self)
@@ -440,8 +449,8 @@ rde_dispatch_imsg(int fd, short event, void *bula)
 				 * in the table we should reset the session.
 				 */
 				if (rde_req_list_exists(nbr, &lsa->hdr)) {
-					imsg_compose(ibuf_ospfe, IMSG_LS_BADREQ,
-					    imsg.hdr.peerid, 0, NULL, 0);
+					imsg_compose_event(iev_ospfe, IMSG_LS_BADREQ,
+					    imsg.hdr.peerid, 0, -1, NULL, 0);
 					break;
 				}
 
@@ -455,13 +464,13 @@ rde_dispatch_imsg(int fd, short event, void *bula)
 					break;
 
 				/* directly send current LSA, no ack */
-				imsg_compose(ibuf_ospfe, IMSG_LS_UPD,
-				    imsg.hdr.peerid, 0, v->lsa,
+				imsg_compose_event(iev_ospfe, IMSG_LS_UPD,
+				    imsg.hdr.peerid, 0, -1, v->lsa,
 				    ntohs(v->lsa->hdr.len));
 			} else {
 				/* LSA equal send direct ack */
-				imsg_compose(ibuf_ospfe, IMSG_LS_ACK,
-				    imsg.hdr.peerid, 0, &lsa->hdr,
+				imsg_compose_event(iev_ospfe, IMSG_LS_ACK,
+				    imsg.hdr.peerid, 0, -1, &lsa->hdr,
 				    sizeof(lsa->hdr));
 				free(lsa);
 			}
@@ -506,8 +515,8 @@ rde_dispatch_imsg(int fd, short event, void *bula)
 			}
 			if (imsg.hdr.len == IMSG_HEADER_SIZE) {
 				LIST_FOREACH(area, &rdeconf->area_list, entry) {
-					imsg_compose(ibuf_ospfe, IMSG_CTL_AREA,
-					    0, imsg.hdr.pid, area,
+					imsg_compose_event(iev_ospfe, IMSG_CTL_AREA,
+					    0, imsg.hdr.pid, -1, area,
 					    sizeof(*area));
 					lsa_dump(&area->lsa_tree, imsg.hdr.type,
 					    imsg.hdr.pid);
@@ -517,8 +526,8 @@ rde_dispatch_imsg(int fd, short event, void *bula)
 			} else {
 				memcpy(&aid, imsg.data, sizeof(aid));
 				if ((area = area_find(rdeconf, aid)) != NULL) {
-					imsg_compose(ibuf_ospfe, IMSG_CTL_AREA,
-					    0, imsg.hdr.pid, area,
+					imsg_compose_event(iev_ospfe, IMSG_CTL_AREA,
+					    0, imsg.hdr.pid, -1, area,
 					    sizeof(*area));
 					lsa_dump(&area->lsa_tree, imsg.hdr.type,
 					    imsg.hdr.pid);
@@ -528,13 +537,13 @@ rde_dispatch_imsg(int fd, short event, void *bula)
 						    imsg.hdr.pid);
 				}
 			}
-			imsg_compose(ibuf_ospfe, IMSG_CTL_END, 0, imsg.hdr.pid,
-			    NULL, 0);
+			imsg_compose_event(iev_ospfe, IMSG_CTL_END, 0, imsg.hdr.pid,
+			    -1, NULL, 0);
 			break;
 		case IMSG_CTL_SHOW_RIB:
 			LIST_FOREACH(area, &rdeconf->area_list, entry) {
-				imsg_compose(ibuf_ospfe, IMSG_CTL_AREA,
-				    0, imsg.hdr.pid, area, sizeof(*area));
+				imsg_compose_event(iev_ospfe, IMSG_CTL_AREA,
+				    0, imsg.hdr.pid, -1, area, sizeof(*area));
 
 				rt_dump(area->id, imsg.hdr.pid, RIB_RTR);
 				rt_dump(area->id, imsg.hdr.pid, RIB_NET);
@@ -542,15 +551,15 @@ rde_dispatch_imsg(int fd, short event, void *bula)
 			aid.s_addr = 0;
 			rt_dump(aid, imsg.hdr.pid, RIB_EXT);
 
-			imsg_compose(ibuf_ospfe, IMSG_CTL_END, 0, imsg.hdr.pid,
-			    NULL, 0);
+			imsg_compose_event(iev_ospfe, IMSG_CTL_END, 0, imsg.hdr.pid,
+			    -1, NULL, 0);
 			break;
 		case IMSG_CTL_SHOW_SUM:
 			rde_send_summary(imsg.hdr.pid);
 			LIST_FOREACH(area, &rdeconf->area_list, entry)
 				rde_send_summary_area(area, imsg.hdr.pid);
-			imsg_compose(ibuf_ospfe, IMSG_CTL_END, 0, imsg.hdr.pid,
-			    NULL, 0);
+			imsg_compose_event(iev_ospfe, IMSG_CTL_END, 0, imsg.hdr.pid,
+			    -1, NULL, 0);
 			break;
 		default:
 			log_debug("rde_dispatch_imsg: unexpected imsg %d",
@@ -560,10 +569,10 @@ rde_dispatch_imsg(int fd, short event, void *bula)
 		imsg_free(&imsg);
 	}
 	if (!shut)
-		imsg_event_add(ibuf);
+		imsg_event_add(iev);
 	else {
 		/* this pipe is dead, so remove the event handler */
-		event_del(&ibuf->ev);
+		event_del(&iev->ev);
 		event_loopexit(NULL);
 	}
 }
@@ -575,29 +584,26 @@ rde_dispatch_parent(int fd, short event, void *bula)
 	static struct area	*narea;
 	struct iface		*niface;
 	struct imsg		 imsg;
-	struct kroute		 kr;
 	struct rroute		 rr;
-	struct imsgbuf		*ibuf = bula;
+	struct imsgev		*iev = bula;
+	struct imsgbuf		*ibuf;
 	struct lsa		*lsa;
 	struct vertex		*v;
-	struct rt_node		*rn;
+	struct redistribute	*nred;
 	ssize_t			 n;
 	int			 shut = 0;
 
-	switch (event) {
-	case EV_READ:
+	ibuf = &iev->ibuf;
+
+	if (event & EV_READ) {
 		if ((n = imsg_read(ibuf)) == -1)
 			fatal("imsg_read error");
 		if (n == 0)	/* connection closed */
 			shut = 1;
-		break;
-	case EV_WRITE:
+	}
+	if (event & EV_WRITE) {
 		if (msgbuf_write(&ibuf->w) == -1)
 			fatal("msgbuf_write");
-		imsg_event_add(ibuf);
-		return;
-	default:
-		fatalx("unknown event");
 	}
 
 	for (;;) {
@@ -642,22 +648,6 @@ rde_dispatch_parent(int fd, short event, void *bula)
 					lsa_merge(nbrself, lsa, v);
 			}
 			break;
-		case IMSG_KROUTE_GET:
-			if (imsg.hdr.len != IMSG_HEADER_SIZE + sizeof(kr)) {
-				log_warnx("rde_dispatch_parent: "
-				    "wrong imsg len");
-				break;
-			}
-			memcpy(&kr, imsg.data, sizeof(kr));
-
-			if ((rn = rt_find(kr.prefix.s_addr, kr.prefixlen,
-			    DT_NET)) != NULL)
-				rde_send_change_kroute(rn);
-			else
-				/* should not happen */
-				imsg_compose(ibuf_main, IMSG_KROUTE_DELETE, 0,
-				    0, &kr, sizeof(kr));
-			break;
 		case IMSG_RECONF_CONF:
 			if ((nconf = malloc(sizeof(struct ospfd_conf))) ==
 			    NULL)
@@ -675,9 +665,17 @@ rde_dispatch_parent(int fd, short event, void *bula)
 			LIST_INIT(&narea->iface_list);
 			LIST_INIT(&narea->nbr_list);
 			RB_INIT(&narea->lsa_tree);
+			SIMPLEQ_INIT(&narea->redist_list);
 
 			LIST_INSERT_HEAD(&nconf->area_list, narea, entry);
 			break;
+		case IMSG_RECONF_REDIST:
+			if ((nred= malloc(sizeof(struct redistribute))) == NULL)
+				fatal(NULL);
+			memcpy(nred, imsg.data, sizeof(struct redistribute));
+
+			SIMPLEQ_INSERT_TAIL(&narea->redist_list, nred, entry);
+			break;
 		case IMSG_RECONF_IFACE:
 			if ((niface = malloc(sizeof(struct iface))) == NULL)
 				fatal(NULL);
@@ -703,10 +701,10 @@ rde_dispatch_parent(int fd, short event, void *bula)
 		imsg_free(&imsg);
 	}
 	if (!shut)
-		imsg_event_add(ibuf);
+		imsg_event_add(iev);
 	else {
 		/* this pipe is dead, so remove the event handler */
-		event_del(&ibuf->ev);
+		event_del(&iev->ev);
 		event_loopexit(NULL);
 	}
 }
@@ -717,6 +715,16 @@ rde_router_id(void)
 	return (rdeconf->rtr_id.s_addr);
 }
 
+struct area *
+rde_backbone_area(void)
+{
+	struct in_addr	id;
+
+	id.s_addr = INADDR_ANY;
+
+	return (area_find(rdeconf, id));
+}
+
 void
 rde_send_change_kroute(struct rt_node *r)
 {
@@ -725,7 +733,7 @@ rde_send_change_kroute(struct rt_node *r)
 	struct rt_nexthop	*rn;
 	struct buf		*wbuf;
 
-	if ((wbuf = imsg_create(ibuf_main, IMSG_KROUTE_CHANGE, 0, 0,
+	if ((wbuf = imsg_create(&iev_main->ibuf, IMSG_KROUTE_CHANGE, 0, 0,
 	    sizeof(kr))) == NULL) {
 		return;
 	}
@@ -744,7 +752,8 @@ rde_send_change_kroute(struct rt_node *r)
 	}
 	if (krcount == 0)
 		fatalx("rde_send_change_kroute: no valid nexthop found");
-	imsg_close(ibuf_main, wbuf);
+	imsg_close(&iev_main->ibuf, wbuf);
+	imsg_event_add(iev_main);
 }
 
 void
@@ -756,7 +765,7 @@ rde_send_delete_kroute(struct rt_node *r)
 	kr.prefix.s_addr = r->prefix.s_addr;
 	kr.prefixlen = r->prefixlen;
 
-	imsg_compose(ibuf_main, IMSG_KROUTE_DELETE, 0, 0, &kr, sizeof(kr));
+	imsg_compose_event(iev_main, IMSG_KROUTE_DELETE, 0, 0, -1, &kr, sizeof(kr));
 }
 
 void
@@ -1064,6 +1073,8 @@ rde_asext_put(struct rroute *rr)
 void
 rde_summary_update(struct rt_node *rte, struct area *area)
 {
+	struct rt_nexthop	*rn;
+	struct rt_node		*nr;
 	struct vertex		*v = NULL;
 	struct lsa		*lsa;
 	u_int8_t		 type = 0;
@@ -1080,7 +1091,16 @@ rde_summary_update(struct rt_node *rte, struct area *area)
 	/* no need to originate inter-area routes to the backbone */
 	if (rte->p_type == PT_INTER_AREA && area->id.s_addr == INADDR_ANY)
 		return;
-	/* TODO nexthop check, nexthop part of area -> no summary */
+	/* nexthop check, nexthop part of area -> no summary */
+	TAILQ_FOREACH(rn, &rte->nexthop, entry) {
+		nr = rt_lookup(DT_NET, rn->nexthop.s_addr);
+		if (nr && nr->area.s_addr == area->id.s_addr)
+			continue;
+		break;
+	}
+	if (rn == NULL)	/* all nexthops belong to this area */
+		return;
+
 	if (rte->cost >= LS_INFINITY)
 		return;
 	/* TODO AS border router specific checks */
@@ -1090,6 +1110,9 @@ rde_summary_update(struct rt_node *rte, struct area *area)
 	if (rte->d_type == DT_NET) {
 		type = LSA_TYPE_SUM_NETWORK;
 	} else if (rte->d_type == DT_RTR) {
+		if (area->stub)
+			/* do not redistribute type 4 LSA into stub areas */
+			return;
 		type = LSA_TYPE_SUM_ROUTER;
 	} else
 		fatalx("rde_summary_update: unknown route type");
